@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 #include <stdlib.h>
 #include <stdio.h>
@@ -43,6 +43,10 @@
 
 #define  SQR(x) ((x) * (x))
 
+#ifndef M_PI
+#define M_PI    3.14159265358979323846
+#endif /* M_PI */
+
 /*  the Blend structures  */
 
 typedef enum
@@ -70,7 +74,7 @@ typedef enum
 {
   REPEAT_NONE,
   REPEAT_SAWTOOTH,
-  REPEAT_TRIANGULAR,
+  REPEAT_TRIANGULAR
 } RepeatMode;
 
 typedef double (*RepeatFunc)(double);
@@ -96,6 +100,7 @@ struct _BlendOptions
   int          paint_mode;
   GradientType gradient_type;
   RepeatMode   repeat;
+  GtkWidget   *repeat_mode_menu;
   int          supersample;
   GtkWidget   *frame;
   int          max_depth;
@@ -136,7 +141,7 @@ static void   blend_motion                  (Tool *, GdkEventMotion *, gpointer)
 static void   blend_cursor_update           (Tool *, GdkEventMotion *, gpointer);
 static void   blend_control                 (Tool *, int, gpointer);
 
-static void   blend                         (GImage *gimage, int drawable_id,
+static void   blend                         (GImage *gimage, GimpDrawable *drawable,
 					     BlendMode blend_mode, int paint_mode,
 					     GradientType gradient_type,
 					     double opacity, double offset,
@@ -165,12 +170,12 @@ static double gradient_repeat_none(double val);
 static double gradient_repeat_sawtooth(double val);
 static double gradient_repeat_triangular(double val);
 
-static void   gradient_precalc_shapeburst   (GImage *gimage, int, PixelRegion *PR, double dist);
+static void   gradient_precalc_shapeburst   (GImage *gimage, GimpDrawable *drawable, PixelRegion *PR, double dist);
 
 static void   gradient_render_pixel(double x, double y, color_t *color, void *render_data);
 static void   gradient_put_pixel(int x, int y, color_t color, void *put_pixel_data);
 
-static void   gradient_fill_region          (GImage *gimage, int, PixelRegion *PR,
+static void   gradient_fill_region          (GImage *gimage, GimpDrawable *drawable, PixelRegion *PR,
 					     int width, int height,
 					     BlendMode blend_mode, GradientType gradient_type,
 					     double offset, RepeatMode repeat,
@@ -246,6 +251,8 @@ gradient_type_callback (GtkWidget *w,
 			gpointer   client_data)
 {
   blend_options->gradient_type = (GradientType) client_data;
+  gtk_widget_set_sensitive (blend_options->repeat_mode_menu, 
+			    (blend_options->gradient_type < 6));
 }
 
 static void
@@ -440,6 +447,8 @@ create_blend_options ()
   gtk_widget_show(label);
   gtk_widget_show(rt_option_menu);
 
+  options->repeat_mode_menu = rt_option_menu;
+
   /* show the whole table */
 
   gtk_widget_show (table);
@@ -541,7 +550,7 @@ blend_button_press (Tool           *tool,
   switch (drawable_type (gimage_active_drawable (gdisp->gimage)))
     {
     case INDEXED_GIMAGE: case INDEXEDA_GIMAGE:
-      message_box ("Blend: Invalid for indexed images.", NULL, NULL);
+      g_message ("Blend: Invalid for indexed images.");
       return;
 
       break;
@@ -594,7 +603,7 @@ blend_button_release (Tool           *tool,
       return_vals = procedural_db_run_proc ("gimp_blend",
 					    &nreturn_vals,
 					    PDB_IMAGE, gimage->ID,
-					    PDB_DRAWABLE, gimage_active_drawable (gimage),
+					    PDB_DRAWABLE, drawable_ID (gimage_active_drawable (gimage)),
 					    PDB_INT32, (gint32) blend_options->blend_mode,
 					    PDB_INT32, (gint32) blend_options->paint_mode,
 					    PDB_INT32, (gint32) blend_options->gradient_type,
@@ -613,7 +622,7 @@ blend_button_release (Tool           *tool,
       if (return_vals[0].value.pdb_int == PDB_SUCCESS)
 	gdisplays_flush ();
       else
-	message_box ("Blend operation failed.", NULL, NULL);
+	g_message ("Blend operation failed.");
 
       procedural_db_destroy_args (return_vals, nreturn_vals);
     }
@@ -726,7 +735,7 @@ blend_control (Tool     *tool,
 /*  The actual blending procedure  */
 static void
 blend (GImage       *gimage,
-       int           drawable_id,
+       GimpDrawable *drawable,
        BlendMode     blend_mode,
        int           paint_mode,
        GradientType  gradient_type,
@@ -748,10 +757,10 @@ blend (GImage       *gimage,
   int bytes;
   int x1, y1, x2, y2;
 
-  has_selection = drawable_mask_bounds (drawable_id, &x1, &y1, &x2, &y2);
+  has_selection = drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
 
-  has_alpha = drawable_has_alpha (drawable_id);
-  bytes = drawable_bytes (drawable_id);
+  has_alpha = drawable_has_alpha (drawable);
+  bytes = drawable_bytes (drawable);
 
   /*  Always create an alpha temp buf (for generality) */
   if (! has_alpha)
@@ -763,7 +772,7 @@ blend (GImage       *gimage,
   buf_tiles = tile_manager_new ((x2 - x1), (y2 - y1), bytes);
   pixel_region_init (&bufPR, buf_tiles, 0, 0, (x2 - x1), (y2 - y1), TRUE);
 
-  gradient_fill_region (gimage, drawable_id,
+  gradient_fill_region (gimage, drawable,
 			&bufPR, (x2 - x1), (y2 - y1),
 			blend_mode, gradient_type, offset, repeat,
 			supersample, max_depth, threshold,
@@ -771,11 +780,11 @@ blend (GImage       *gimage,
 			(endx - x1), (endy - y1));
 
   pixel_region_init (&bufPR, buf_tiles, 0, 0, (x2 - x1), (y2 - y1), FALSE);
-  gimage_apply_image (gimage, drawable_id, &bufPR, TRUE,
+  gimage_apply_image (gimage, drawable, &bufPR, TRUE,
 		      (opacity * 255) / 100, paint_mode, NULL, x1, y1);
 
   /*  update the image  */
-  drawable_update (drawable_id, x1, y1, (x2 - x1), (y2 - y1));
+  drawable_update (drawable, x1, y1, (x2 - x1), (y2 - y1));
 
   /*  free the temporary buffer  */
   tile_manager_destroy (buf_tiles);
@@ -891,6 +900,8 @@ gradient_calc_square_factor (double dist,
 
       if (rat < offset)
 	rat = 0.0;
+      else if (offset == 1)
+        rat = (rat>=1) ? 1 : 0;
       else
 	rat = (rat - offset) / (1.0 - offset);
     } /* else */
@@ -923,6 +934,8 @@ gradient_calc_radial_factor (double dist,
 
       if (rat < offset)
 	rat = 0.0;
+      else if (offset == 1)
+        rat = (rat>=1) ? 1 : 0;
       else
 	rat = (rat - offset) / (1.0 - offset);
     } /* else */
@@ -979,6 +992,8 @@ gradient_calc_bilinear_factor (double  dist,
 
       if (fabs(rat) < offset)
 	rat = 0.0;
+      else if (offset == 1)
+        rat = (rat>=1) ? 1 : 0;
       else
 	rat = (fabs(rat) - offset) / (1.0 - offset);
     } /* else */
@@ -1079,7 +1094,7 @@ gradient_repeat_triangular(double val)
 /*****/
 static void
 gradient_precalc_shapeburst (GImage      *gimage,
-			     int          drawable_id,
+			     GimpDrawable *drawable,
 			     PixelRegion *PR,
 			     double       dist)
 {
@@ -1089,7 +1104,7 @@ gradient_precalc_shapeburst (GImage      *gimage,
   float *distp;
   int size;
   void * pr;
-  unsigned char white[1] = { OPAQUE };
+  unsigned char white[1] = { OPAQUE_OPACITY };
 
   /*  allocate the distance map  */
   if (distR.tiles)
@@ -1107,12 +1122,13 @@ gradient_precalc_shapeburst (GImage      *gimage,
       int x1, y1, x2, y2;
       int offx, offy;
 
-      drawable_mask_bounds (drawable_id, &x1, &y1, &x2, &y2);
-      drawable_offsets (drawable_id, &offx, &offy);
+      drawable_mask_bounds (drawable, &x1, &y1, &x2, &y2);
+      drawable_offsets (drawable, &offx, &offy);
 
       /*  the selection mask  */
       mask = gimage_get_mask (gimage);
-      pixel_region_init (&maskR, mask->tiles, x1 + offx, y1 + offy, (x2 - x1), (y2 - y1), FALSE);
+      pixel_region_init (&maskR, drawable_data (GIMP_DRAWABLE(mask)), 
+			 x1 + offx, y1 + offy, (x2 - x1), (y2 - y1), FALSE);
 
       /*  copy the mask to the temp mask  */
       copy_region (&maskR, &tempR);
@@ -1121,11 +1137,11 @@ gradient_precalc_shapeburst (GImage      *gimage,
   else
     {
       /*  If the intended drawable has an alpha channel, use that  */
-      if (drawable_has_alpha (drawable_id))
+      if (drawable_has_alpha (drawable))
 	{
 	  PixelRegion drawableR;
 
-	  pixel_region_init (&drawableR, drawable_data (drawable_id), PR->x, PR->y, PR->w, PR->h, FALSE);
+	  pixel_region_init (&drawableR, drawable_data (drawable), PR->x, PR->y, PR->w, PR->h, FALSE);
 
 	  extract_alpha_region (&drawableR, NULL, &tempR);
 	}
@@ -1280,7 +1296,7 @@ gradient_put_pixel(int x, int y, color_t color, void *put_pixel_data)
 
 static void
 gradient_fill_region (GImage       *gimage,
-		      int           drawable_id,
+		      GimpDrawable *drawable,
 		      PixelRegion  *PR,
 		      int           width,
 		      int           height,
@@ -1381,7 +1397,7 @@ gradient_fill_region (GImage       *gimage,
     case ShapeburstSpherical:
     case ShapeburstDimpled:
       rbd.dist = sqrt(SQR(ex - sx) + SQR(ey - sy));
-      gradient_precalc_shapeburst(gimage, drawable_id, PR, rbd.dist);
+      gradient_precalc_shapeburst(gimage, drawable, PR, rbd.dist);
       break;
 
     default:
@@ -1646,6 +1662,7 @@ tools_new_blend ()
   tool->arrow_keys_func = standard_arrow_keys_func;
   tool->cursor_update_func = blend_cursor_update;
   tool->control_func = blend_control;
+  tool->preserve = TRUE;
 
   return tool;
 }
@@ -1763,7 +1780,7 @@ blend_invoker (Argument *args)
 {
   int success = TRUE;
   GImage *gimage;
-  int drawable_id;
+  GimpDrawable *drawable;
   BlendMode blend_mode;
   int paint_mode;
   GradientType gradient_type;
@@ -1778,6 +1795,17 @@ blend_invoker (Argument *args)
   int int_value;
   double fp_value;
 
+  drawable    = NULL;
+  blend_mode  = FG_BG_RGB_MODE;
+  paint_mode  = NORMAL_MODE;
+  gradient_type = Linear;
+  opacity       = 100.0;
+  offset        = 0.0;
+  repeat        = REPEAT_NONE;
+  supersample   = FALSE;
+  max_depth     = 0;
+  threshold     = 0.0;
+
   /*  the gimage  */
   if (success)
     {
@@ -1789,9 +1817,8 @@ blend_invoker (Argument *args)
   if (success)
     {
       int_value = args[1].value.pdb_int;
-      if (gimage == drawable_gimage (int_value))
-	drawable_id = int_value;
-      else
+      drawable = drawable_get_ID (int_value);
+      if (drawable == NULL || gimage != drawable_gimage (drawable))
 	success = FALSE;
     }
   /*  blend mode  */
@@ -1903,7 +1930,7 @@ blend_invoker (Argument *args)
   /*  call the blend procedure  */
   if (success)
     {
-      blend (gimage, drawable_id, blend_mode, paint_mode, gradient_type,
+      blend (gimage, drawable, blend_mode, paint_mode, gradient_type,
 	     opacity, offset, repeat, supersample, max_depth, threshold, x1, y1, x2, y2);
     }
 
