@@ -50,6 +50,10 @@
 #include <libgimp/gimpui.h>
 #include <string.h>
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 /*
  * Constants...
  */
